#include <stdlib.h>
#include <string.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
#define YYPREFIX "yy"
#line 2 "parse.y"

/*
 * Copyright (c) 2021, 2022 Omar Polo <op@omarpolo.com>
 * Copyright (c) 2018 Florian Obser <florian@openbsd.org>
 * Copyright (c) 2004, 2005 Esben Norby <norby@openbsd.org>
 * Copyright (c) 2004 Ryan McBride <mcbride@openbsd.org>
 * Copyright (c) 2002, 2003, 2004 Henning Brauer <henning@openbsd.org>
 * Copyright (c) 2001 Markus Friedl.  All rights reserved.
 * Copyright (c) 2001 Daniel Hartmeier.  All rights reserved.
 * Copyright (c) 2001 Theo de Raadt.  All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "gmid.h"

#include <ctype.h>
#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

TAILQ_HEAD(files, file)		 files = TAILQ_HEAD_INITIALIZER(files);
static struct file {
	TAILQ_ENTRY(file)	 entry;
	FILE			*stream;
	char			*name;
	size_t	 		 ungetpos;
	size_t			 ungetsize;
	u_char			*ungetbuf;
	int			 eof_reached;
	int			 lineno;
	int			 errors;
} *file, *topfile;

struct file	*pushfile(const char *, int);
int		 popfile(void);
int		 yyparse(void);
int		 yylex(void);
void		 yyerror(const char *, ...)
    __attribute__((__format__ (printf, 1, 2)))
    __attribute__((__nonnull__ (1)));
void		 yywarn(const char *, ...)
    __attribute__((__format__ (printf, 1, 2)))
    __attribute__((__nonnull__ (1)));
int		 kw_cmp(const void *, const void *);
int		 lookup(char *);
int		 igetc(void);
int		 lgetc(int);
void		 lungetc(int);
int		 findeol(void);

/*
 * #define YYDEBUG 1
 * int yydebug = 1;
 */

TAILQ_HEAD(symhead, sym) symhead = TAILQ_HEAD_INITIALIZER(symhead);
struct sym {
	TAILQ_ENTRY(sym)	 entry;
	int			 used;
	int			 persist;
	char			*name;
	char			*val;
};

int	 symset(const char *, const char *, int);
char	*symget(const char *);

struct vhost	*new_vhost(void);
struct location	*new_location(void);
struct proxy	*new_proxy(void);
char		*ensure_absolute_path(char*);
int		 check_block_code(int);
char		*check_block_fmt(char*);
int		 check_strip_no(int);
int		 check_port_num(int);
int		 check_prefork_num(int);
void		 advance_loc(void);
void		 advance_proxy(void);
void		 parsehp(char *, char **, const char **, const char *);
void		 only_once(const void*, const char*);
void		 only_oncei(int, const char*);
int		 fastcgi_conf(char *, char *, char *);
void		 add_param(char *, char *, int);

static struct vhost		*host;
static struct location		*loc;
static struct proxy		*proxy;
static char			*current_media;
static int			 errors;

typedef struct {
	union {
		char	*string;
		int	 number;
	} v;
	int lineno;
} YYSTYPE;

#line 125 "y.tab.c"
#define ALIAS 257
#define AUTO 258
#define BLOCK 259
#define CA 260
#define CERT 261
#define CGI 262
#define CHROOT 263
#define CLIENT 264
#define DEFAULT 265
#define ENTRYPOINT 266
#define ENV 267
#define FASTCGI 268
#define FOR_HOST 269
#define INCLUDE 270
#define INDEX 271
#define IPV6 272
#define KEY 273
#define LANG 274
#define LOCATION 275
#define LOG 276
#define MAP 277
#define MIME 278
#define OCSP 279
#define OFF 280
#define ON 281
#define PARAM 282
#define PORT 283
#define PREFORK 284
#define PROTO 285
#define PROTOCOLS 286
#define PROXY 287
#define RELAY_TO 288
#define REQUIRE 289
#define RETURN 290
#define ROOT 291
#define SERVER 292
#define SNI 293
#define SPAWN 294
#define STRIP 295
#define TCP 296
#define TOEXT 297
#define TYPE 298
#define TYPES 299
#define USE_TLS 300
#define USER 301
#define VERIFYNAME 302
#define ERROR 303
#define STRING 304
#define NUM 305
#define YYERRCODE 256
const short yylhs[] =
	{                                        -1,
    0,    0,    0,    0,    0,    0,    0,    0,    4,    1,
    1,    2,    2,    3,    3,    5,    6,    6,    6,    6,
    6,    6,    6,    6,    9,    7,    7,   11,   11,   11,
   11,   12,   12,   12,   12,   12,   12,   12,   12,   12,
   17,   14,   16,   16,   19,   19,   18,   18,   20,   20,
   20,   20,   20,   20,   20,   20,   21,   13,   13,   22,
   22,   15,   15,   15,   15,   15,   15,   15,   15,   15,
   15,   15,   15,   23,   23,   23,   23,   23,    8,   24,
   24,   28,   25,   25,   27,   27,   30,   26,   10,   10,
   10,   29,   29,
};
const short yylen[] =
	{                                         2,
    0,    3,    2,    3,    3,    3,    3,    3,    2,    1,
    1,    2,    1,    1,    1,    3,    2,    2,    3,    4,
    2,    2,    2,    2,    0,    7,    2,    0,    3,    3,
    3,    2,    2,    2,    2,    4,    2,    2,    4,    1,
    0,    7,    0,    2,    2,    2,    0,    3,    2,    2,
    2,    2,    4,    2,    2,    2,    0,    7,    2,    0,
    3,    3,    4,    3,    1,    3,    2,    2,    2,    2,
    4,    2,    2,    2,    1,    4,    2,    4,    5,    3,
    2,    0,    4,    1,    2,    1,    1,    2,    2,    2,
    0,    1,    0,
};
const short yydefred[] =
	{                                      1,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    3,    0,    0,    0,    0,    0,
    8,   27,   13,    0,    9,   11,   10,   18,    0,    0,
   21,   22,    0,    0,    0,    0,    0,    2,    4,    5,
    6,    7,   12,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   89,   90,   82,   84,    0,    0,   28,    0,
   79,    0,    0,   81,    0,   15,   14,   87,    0,   86,
   80,   88,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   57,    0,    0,    0,   41,
    0,    0,    0,   26,    0,    0,    0,   40,   92,   83,
   85,   59,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   67,    0,    0,    0,    0,   70,    0,
    0,   43,    0,    0,   73,   29,   30,   31,   62,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   44,    0,   76,    0,
   60,    0,    0,   47,    0,    0,   58,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   42,    0,   61,    0,
    0,    0,    0,    0,    0,   55,   56,   48,    0,    0,
};
const short yydgoto[] =
	{                                       1,
   28,   24,   68,   56,   17,   18,   19,   20,   46,   49,
   65,   95,   96,   97,   98,  137,  122,  156,  147,  168,
  118,  155,  114,   57,   58,   64,   69,   60,  100,   70,
};
const short yysindex[] =
	{                                      0,
  -10,    1, -281, -272, -245, -281, -265, -264, -263, -281,
 -281,  -83, -281,  -18,    0,   34,   35,   38,   41,   42,
    0,    0,    0, -251,    0,    0,    0,    0, -277, -281,
    0,    0, -251, -251,    2, -251, -281,    0,    0,    0,
    0,    0,    0, -281, -251,  -69,    2,    2, -255, -251,
 -251,    2,    0,    0,    0,    0, -121,   45,    0, -279,
    0,   45,    2,    0, 1188,    0,    0,    0,  -57,    0,
    0,    0,  -68, -281, -213, -231, -281, -281, -238, -281,
 -281, -275, -281, -281, -281,    0, -245, -281, -281,    0,
 -202, -281, -242,    0,    2,    2,    2,    0,    0,    0,
    0,    0, -251, -245, -237, -251, -251, -281, -251,  -60,
 -281, -281, -251,    0, -251, -251, -251, -281,    0, -251,
  -55,    0, -191, -251,    0,    0,    0,    0,    0, -281,
 -251, -281, -251, -270, -120, -281, -118, -281, -251, -251,
 -267,    2, -251, -281, -281,    2,    0, -251,    0, -251,
    0, -251, -251,    0,  -34,  -97,    0,    2, -281, -281,
 -281, -281, -193, -281, -245, -245,    0,    2,    0, -251,
 -251, -251, -251, -188, -251,    0,    0,    0, -281, -251,};
const short yyrindex[] =
	{                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   64,    0,    0,    0,    0,    0,    0,
    0,    0,   65,  -47, -254,   67,    0,    0,    0,    0,
    0,    0,    0,    0,   68,    0,  993,  993,    0,   70,
   71, 1229,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, -117,    0,    0,    0,    0,    0,   72,    0,
    0,    0,    0,    0,    0,  107,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1229, 1229, 1229,    0,    0,    0,
    0,    0,  147,    0,    0,  192,  233,    0,  285,    0,
    0,    0,  331,    0,  378,  423,  466,    0,    0,  506,
    0,    0,    0,  552,    0,    0,    0,    0,    0,  604,
  645,    0,  690,  733,    0,    0,    0,    0,  773,  814,
    0, 1056,  858,    0,    0,  -79,    0,  912,    0,  952,
    0, -114, -113,    0,    0,    0,    0, 1056,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  -79,    0,   -3,
   14,   20,   37,    0,   54,    0,    0,    0,    0,   60,};
const short yygindex[] =
	{                                      0,
  -73, 1279,    0,   82,    0,    0,    0,    0,    0,  -30,
    0,    0,    0,    0,  -71,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   28,   24,    0,    0,    0,   18,
};
#define YYTABLESIZE 1524
const short yytable[] =
	{                                      15,
  132,   99,  142,   61,  146,  136,   49,   91,   46,   45,
   21,   47,  141,  119,    4,   91,   53,   54,  111,   44,
  112,   59,   23,   50,   66,   67,   43,  167,   23,   51,
  129,   25,   72,   43,   26,   27,   23,  149,   30,   35,
   31,   32,   37,   38,   39,   91,   52,   40,   55,   91,
   41,   42,   43,   52,   63,   49,  102,  104,  105,  108,
   48,  123,  125,   54,  126,  127,  128,  130,  138,   53,
  174,  179,   50,   17,   23,   25,   24,   19,   51,   16,
   20,   93,   16,  158,   62,   71,  101,    0,    0,    0,
  157,  176,  177,    0,    0,   52,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  151,   54,    0,    0,  154,   65,    0,   53,    0,
    0,   49,    0,    0,    0,   22,    0,  169,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  178,   50,    0,
    0,    0,    0,    0,   51,    0,    0,    0,    4,    0,
  144,    0,   91,    0,   46,   45,   32,    0,    0,    0,
    0,   52,    0,  159,    0,   65,  145,    0,    0,    0,
   46,   45,    0,    0,    0,  160,    0,    0,   54,    0,
    0,   91,   55,   43,   53,    0,   91,    0,  161,    0,
  162,  163,    0,   91,    0,  164,    0,    0,    0,    0,
    0,   33,  165,    0,  166,   32,   91,    0,   91,   91,
    0,    0,    0,   91,    0,    0,    0,    0,    0,    0,
   91,    0,   91,   75,   76,    0,    0,    0,    0,    0,
   79,   65,    0,   82,    0,    0,   83,    0,    0,   85,
    0,   87,   34,   43,    0,    2,   66,   67,   43,    0,
   33,    0,    3,    0,   91,    0,   92,   49,    0,    4,
   93,    5,    0,    0,    0,    0,    6,    7,    0,   49,
    0,   32,    8,    9,   50,   10,    0,    0,    0,    0,
   51,   11,   49,    0,   49,   49,   50,    0,   12,   49,
   13,   34,   51,   14,   35,    0,   49,   52,   49,   50,
    0,   50,   50,    0,    0,   51,   50,   51,   51,   52,
    0,    0,   51,   50,   54,   50,   33,    0,    0,   51,
   53,   51,   52,    0,   52,   52,   54,    0,    0,   52,
    0,    0,   53,    0,    0,    0,   52,    0,   52,   54,
   75,   54,   54,   35,    0,   53,   54,   53,   53,    0,
    0,    0,   53,   54,    0,   54,    0,   34,    0,   53,
    0,   53,   65,   65,   65,   65,    0,   65,   65,    0,
    0,   65,   65,   65,   65,    0,    0,   65,    0,   65,
   65,   65,   65,    0,    0,   65,    0,   68,   65,   75,
    0,    0,    0,   65,    0,   65,    0,   65,    0,    0,
    0,   65,   32,   32,   32,   32,    0,   32,   32,   35,
    0,   32,   32,   32,   32,    0,    0,   32,    0,   32,
   32,   32,   32,    0,    0,   32,    0,    0,   32,    0,
    0,    0,   37,   32,    0,   32,   68,   32,    0,    0,
    0,   32,    0,    0,    0,    0,    0,   33,   33,   33,
   33,    0,   33,   33,    0,   75,   33,   33,   33,   33,
    0,    0,   33,    0,   33,   33,   33,   33,    0,    0,
   33,    0,    0,   33,    0,   69,    0,    0,   33,    0,
   33,   37,   33,    0,    0,    0,   33,    0,   34,   34,
   34,   34,    0,   34,   34,    0,    0,   34,   34,   34,
   34,    0,   68,   34,    0,   34,   34,   34,   34,    0,
    0,   34,    0,    0,   34,   38,    0,    0,    0,   34,
    0,   34,    0,   34,   69,    0,    0,   34,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   35,   35,   35,   35,    0,   35,   35,   37,    0,   35,
   35,   35,   35,    0,    0,   35,    0,   35,   35,   35,
   35,   72,    0,   35,   38,    0,   35,    0,    0,    0,
    0,   35,    0,   35,    0,   35,    0,    0,    0,   35,
    0,    0,    0,    0,    0,    0,   75,   75,   75,   75,
   69,   75,   75,    0,    0,   75,   75,   75,   75,    0,
    0,   75,    0,   75,   75,   75,   75,    0,    0,   75,
   72,    0,   75,   64,    0,    0,    0,   75,    0,   75,
    0,   75,    0,    0,    0,   75,    0,    0,    0,    0,
   38,    0,    0,   68,   68,   68,   68,    0,   68,   68,
    0,    0,   68,   68,   68,   68,    0,    0,   68,    0,
   68,   68,   68,   68,   66,    0,   68,    0,    0,   68,
    0,    0,   64,    0,   68,    0,   68,    0,   68,    0,
    0,    0,   68,    0,    0,    0,   72,    0,   37,   37,
   37,   37,    0,   37,   37,    0,    0,   37,   37,   37,
   37,    0,    0,   37,    0,   37,   37,   37,   37,   74,
    0,   37,    0,   66,   37,    0,    0,    0,    0,   37,
    0,   37,    0,   37,    0,    0,    0,   37,    0,    0,
    0,   69,   69,   69,   69,    0,   69,   69,   64,    0,
   69,   69,   69,   69,    0,    0,   69,    0,   69,   69,
   69,   69,   77,    0,   69,    0,    0,   69,   74,    0,
    0,    0,   69,    0,   69,    0,   69,    0,    0,    0,
   69,   38,   38,   38,   38,    0,   38,   38,    0,   66,
   38,   38,   38,   38,    0,    0,   38,    0,   38,   38,
   38,   38,   63,    0,   38,    0,    0,   38,    0,    0,
    0,   77,   38,    0,   38,    0,   38,    0,    0,    0,
   38,    0,    0,    0,    0,    0,    0,   72,   72,   72,
   72,    0,   72,   72,   74,    0,   72,   72,   72,   72,
    0,    0,   72,   36,   72,   72,   72,   72,    0,    0,
   72,   63,    0,   72,    0,    0,    0,    0,   72,    0,
   72,    0,   72,    0,    0,    0,   72,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   77,    0,   64,
   64,   64,   64,    0,   64,   64,    0,   39,   64,   64,
   64,   64,   36,    0,   64,    0,   64,   64,   64,   64,
    0,    0,   64,    0,    0,   64,    0,    0,    0,    0,
   64,    0,   64,    0,   64,    0,    0,   63,   64,    0,
   66,   66,   66,   66,    0,   66,   66,    0,    0,   66,
   66,   66,   66,    0,    0,   66,   39,   66,   66,   66,
   66,   71,    0,   66,    0,    0,   66,    0,    0,    0,
    0,   66,    0,   66,    0,   66,    0,    0,   36,   66,
    0,    0,    0,    0,    0,   74,   74,   74,   74,    0,
   74,   74,    0,    0,   74,   74,   74,   74,    0,    0,
   74,   78,   74,   74,   74,   74,    0,    0,   74,    0,
   71,   74,    0,    0,    0,    0,   74,    0,   74,    0,
   74,    0,   39,    0,   74,    0,    0,    0,   77,   77,
   77,   77,    0,   77,   77,    0,    0,   77,   77,   77,
   77,    0,    0,   77,    0,   77,   77,   77,   77,    0,
   78,   77,    0,    0,   77,    0,    0,    0,    0,   77,
    0,   77,    0,   77,    0,    0,    0,   77,   63,   63,
   63,   63,    0,   63,   63,    0,   71,   63,   63,   63,
   63,    0,    0,   63,    0,   63,   63,   63,   63,    0,
    0,   63,    0,    0,   63,    0,    0,    0,    0,   63,
    0,   63,    0,   63,    0,    0,    0,   63,    0,   36,
   36,   36,   36,    0,   36,   36,   78,    0,   36,   36,
   36,   36,    0,    0,   36,    0,   36,   36,   36,   36,
    0,    0,   36,    0,    0,   36,    0,    0,    0,    0,
   36,    0,   36,    0,   36,    0,    0,    0,   36,    0,
    0,    0,    0,   39,   39,   39,   39,   91,   39,   39,
    0,    0,   39,   39,   39,   39,    0,    0,   39,    0,
   39,   39,   39,   39,    0,    0,   39,    0,    0,   39,
    0,    0,    0,    0,   39,    0,   39,    0,   39,    0,
    0,    0,   39,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   71,   71,   71,
   71,    0,   71,   71,    0,    0,   71,   71,   71,   71,
   91,    0,   71,    0,   71,   71,   71,   71,    0,    0,
   71,    0,    0,   71,    0,    0,    0,    0,   71,    0,
   71,    0,   71,    0,    0,    0,   71,   78,   78,   78,
   78,    0,   78,   78,    0,    0,   78,   78,   78,   78,
    0,    0,   78,    0,   78,   78,   78,   78,    0,    0,
   78,    0,    0,   78,    0,    0,    0,    0,   78,    0,
   78,    0,   78,    0,    0,    0,   78,    0,   91,   91,
   91,   91,    0,   91,   91,    0,    0,   91,   91,   91,
   91,    0,   91,   91,    0,   91,   91,   91,   91,    0,
    0,   91,    0,    0,   91,    0,    0,    0,   91,   91,
   91,   91,    0,   91,   29,   91,    0,   91,   33,   34,
    0,   36,   91,    0,   91,    0,   91,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   45,    0,
    0,    0,   94,   91,   91,   50,    0,    0,    0,    0,
   91,    0,   51,   91,    0,    0,   91,    0,    0,   91,
    0,   91,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   91,    0,   91,    0,    0,    0,
   91,    0,  103,   91,    0,  106,  107,    0,  109,  110,
  113,  115,  116,  117,    0,    0,  120,  121,    0,    0,
  124,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  131,    0,    0,  133,
  134,    0,    0,    0,    0,    0,  135,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  139,    0,
  140,    0,    0,    0,  143,    0,  148,    0,    0,  150,
    0,    0,  152,  153,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  170,  171,  172,
  173,    0,  175,   73,   74,   75,   76,    0,   77,   78,
    0,    0,   79,   80,   81,   82,    0,  180,   83,    0,
   84,   85,   86,   87,    0,    0,   88,    0,    0,   89,
    0,    0,    0,    0,   90,    0,   91,    0,   92,    0,
    0,    0,   93,    0,   91,   91,   91,   91,    0,   91,
   91,    0,    0,   91,   91,   91,   91,    0,    0,   91,
    0,   91,   91,   91,   91,    0,    0,   91,    0,    0,
   91,    0,    0,    0,    0,   91,    0,   91,    0,   91,
    0,    0,    0,   91,
};
const short yycheck[] =
	{                                      10,
   61,   59,  123,  125,  123,   61,   10,  125,  123,  123,
   10,   10,  283,   87,  270,  270,   47,   48,  294,  297,
  296,   52,  304,   10,  304,  305,  304,  125,  304,   10,
  104,  304,   63,  304,  280,  281,  304,  305,  304,  123,
  305,  305,   61,   10,   10,  125,   10,   10,  304,  304,
   10,   10,  304,  123,   10,   59,  125,  271,  290,  298,
   59,  264,  305,   10,   95,   96,   97,  305,  260,   10,
  264,  260,   59,   10,   10,  123,   10,   10,   59,   10,
   10,   10,    1,  155,   57,   62,   69,   -1,   -1,   -1,
  125,  165,  166,   -1,   -1,   59,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  142,   59,   -1,   -1,  146,   10,   -1,   59,   -1,
   -1,  125,   -1,   -1,   -1,  125,   -1,  158,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  168,  125,   -1,
   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,  270,   -1,
  269,   -1,  270,   -1,  269,  269,   10,   -1,   -1,   -1,
   -1,  125,   -1,  261,   -1,   59,  285,   -1,   -1,   -1,
  285,  285,   -1,   -1,   -1,  273,   -1,   -1,  125,   -1,
   -1,  261,  304,  304,  125,   -1,  304,   -1,  286,   -1,
  288,  289,   -1,  273,   -1,  293,   -1,   -1,   -1,   -1,
   -1,   10,  300,   -1,  302,   59,  286,   -1,  288,  289,
   -1,   -1,   -1,  293,   -1,   -1,   -1,   -1,   -1,   -1,
  300,   -1,  302,  258,  259,   -1,   -1,   -1,   -1,   -1,
  265,  125,   -1,  268,   -1,   -1,  271,   -1,   -1,  274,
   -1,  276,   10,  304,   -1,  256,  304,  305,  304,   -1,
   59,   -1,  263,   -1,  289,   -1,  291,  261,   -1,  270,
  295,  272,   -1,   -1,   -1,   -1,  277,  278,   -1,  273,
   -1,  125,  283,  284,  261,  286,   -1,   -1,   -1,   -1,
  261,  292,  286,   -1,  288,  289,  273,   -1,  299,  293,
  301,   59,  273,  304,   10,   -1,  300,  261,  302,  286,
   -1,  288,  289,   -1,   -1,  286,  293,  288,  289,  273,
   -1,   -1,  293,  300,  261,  302,  125,   -1,   -1,  300,
  261,  302,  286,   -1,  288,  289,  273,   -1,   -1,  293,
   -1,   -1,  273,   -1,   -1,   -1,  300,   -1,  302,  286,
   10,  288,  289,   59,   -1,  286,  293,  288,  289,   -1,
   -1,   -1,  293,  300,   -1,  302,   -1,  125,   -1,  300,
   -1,  302,  256,  257,  258,  259,   -1,  261,  262,   -1,
   -1,  265,  266,  267,  268,   -1,   -1,  271,   -1,  273,
  274,  275,  276,   -1,   -1,  279,   -1,   10,  282,   59,
   -1,   -1,   -1,  287,   -1,  289,   -1,  291,   -1,   -1,
   -1,  295,  256,  257,  258,  259,   -1,  261,  262,  125,
   -1,  265,  266,  267,  268,   -1,   -1,  271,   -1,  273,
  274,  275,  276,   -1,   -1,  279,   -1,   -1,  282,   -1,
   -1,   -1,   10,  287,   -1,  289,   59,  291,   -1,   -1,
   -1,  295,   -1,   -1,   -1,   -1,   -1,  256,  257,  258,
  259,   -1,  261,  262,   -1,  125,  265,  266,  267,  268,
   -1,   -1,  271,   -1,  273,  274,  275,  276,   -1,   -1,
  279,   -1,   -1,  282,   -1,   10,   -1,   -1,  287,   -1,
  289,   59,  291,   -1,   -1,   -1,  295,   -1,  256,  257,
  258,  259,   -1,  261,  262,   -1,   -1,  265,  266,  267,
  268,   -1,  125,  271,   -1,  273,  274,  275,  276,   -1,
   -1,  279,   -1,   -1,  282,   10,   -1,   -1,   -1,  287,
   -1,  289,   -1,  291,   59,   -1,   -1,  295,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  256,  257,  258,  259,   -1,  261,  262,  125,   -1,  265,
  266,  267,  268,   -1,   -1,  271,   -1,  273,  274,  275,
  276,   10,   -1,  279,   59,   -1,  282,   -1,   -1,   -1,
   -1,  287,   -1,  289,   -1,  291,   -1,   -1,   -1,  295,
   -1,   -1,   -1,   -1,   -1,   -1,  256,  257,  258,  259,
  125,  261,  262,   -1,   -1,  265,  266,  267,  268,   -1,
   -1,  271,   -1,  273,  274,  275,  276,   -1,   -1,  279,
   59,   -1,  282,   10,   -1,   -1,   -1,  287,   -1,  289,
   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,   -1,
  125,   -1,   -1,  256,  257,  258,  259,   -1,  261,  262,
   -1,   -1,  265,  266,  267,  268,   -1,   -1,  271,   -1,
  273,  274,  275,  276,   10,   -1,  279,   -1,   -1,  282,
   -1,   -1,   59,   -1,  287,   -1,  289,   -1,  291,   -1,
   -1,   -1,  295,   -1,   -1,   -1,  125,   -1,  256,  257,
  258,  259,   -1,  261,  262,   -1,   -1,  265,  266,  267,
  268,   -1,   -1,  271,   -1,  273,  274,  275,  276,   10,
   -1,  279,   -1,   59,  282,   -1,   -1,   -1,   -1,  287,
   -1,  289,   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,
   -1,  256,  257,  258,  259,   -1,  261,  262,  125,   -1,
  265,  266,  267,  268,   -1,   -1,  271,   -1,  273,  274,
  275,  276,   10,   -1,  279,   -1,   -1,  282,   59,   -1,
   -1,   -1,  287,   -1,  289,   -1,  291,   -1,   -1,   -1,
  295,  256,  257,  258,  259,   -1,  261,  262,   -1,  125,
  265,  266,  267,  268,   -1,   -1,  271,   -1,  273,  274,
  275,  276,   10,   -1,  279,   -1,   -1,  282,   -1,   -1,
   -1,   59,  287,   -1,  289,   -1,  291,   -1,   -1,   -1,
  295,   -1,   -1,   -1,   -1,   -1,   -1,  256,  257,  258,
  259,   -1,  261,  262,  125,   -1,  265,  266,  267,  268,
   -1,   -1,  271,   10,  273,  274,  275,  276,   -1,   -1,
  279,   59,   -1,  282,   -1,   -1,   -1,   -1,  287,   -1,
  289,   -1,  291,   -1,   -1,   -1,  295,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  125,   -1,  256,
  257,  258,  259,   -1,  261,  262,   -1,   10,  265,  266,
  267,  268,   59,   -1,  271,   -1,  273,  274,  275,  276,
   -1,   -1,  279,   -1,   -1,  282,   -1,   -1,   -1,   -1,
  287,   -1,  289,   -1,  291,   -1,   -1,  125,  295,   -1,
  256,  257,  258,  259,   -1,  261,  262,   -1,   -1,  265,
  266,  267,  268,   -1,   -1,  271,   59,  273,  274,  275,
  276,   10,   -1,  279,   -1,   -1,  282,   -1,   -1,   -1,
   -1,  287,   -1,  289,   -1,  291,   -1,   -1,  125,  295,
   -1,   -1,   -1,   -1,   -1,  256,  257,  258,  259,   -1,
  261,  262,   -1,   -1,  265,  266,  267,  268,   -1,   -1,
  271,   10,  273,  274,  275,  276,   -1,   -1,  279,   -1,
   59,  282,   -1,   -1,   -1,   -1,  287,   -1,  289,   -1,
  291,   -1,  125,   -1,  295,   -1,   -1,   -1,  256,  257,
  258,  259,   -1,  261,  262,   -1,   -1,  265,  266,  267,
  268,   -1,   -1,  271,   -1,  273,  274,  275,  276,   -1,
   59,  279,   -1,   -1,  282,   -1,   -1,   -1,   -1,  287,
   -1,  289,   -1,  291,   -1,   -1,   -1,  295,  256,  257,
  258,  259,   -1,  261,  262,   -1,  125,  265,  266,  267,
  268,   -1,   -1,  271,   -1,  273,  274,  275,  276,   -1,
   -1,  279,   -1,   -1,  282,   -1,   -1,   -1,   -1,  287,
   -1,  289,   -1,  291,   -1,   -1,   -1,  295,   -1,  256,
  257,  258,  259,   -1,  261,  262,  125,   -1,  265,  266,
  267,  268,   -1,   -1,  271,   -1,  273,  274,  275,  276,
   -1,   -1,  279,   -1,   -1,  282,   -1,   -1,   -1,   -1,
  287,   -1,  289,   -1,  291,   -1,   -1,   -1,  295,   -1,
   -1,   -1,   -1,  256,  257,  258,  259,  125,  261,  262,
   -1,   -1,  265,  266,  267,  268,   -1,   -1,  271,   -1,
  273,  274,  275,  276,   -1,   -1,  279,   -1,   -1,  282,
   -1,   -1,   -1,   -1,  287,   -1,  289,   -1,  291,   -1,
   -1,   -1,  295,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  256,  257,  258,
  259,   -1,  261,  262,   -1,   -1,  265,  266,  267,  268,
  125,   -1,  271,   -1,  273,  274,  275,  276,   -1,   -1,
  279,   -1,   -1,  282,   -1,   -1,   -1,   -1,  287,   -1,
  289,   -1,  291,   -1,   -1,   -1,  295,  256,  257,  258,
  259,   -1,  261,  262,   -1,   -1,  265,  266,  267,  268,
   -1,   -1,  271,   -1,  273,  274,  275,  276,   -1,   -1,
  279,   -1,   -1,  282,   -1,   -1,   -1,   -1,  287,   -1,
  289,   -1,  291,   -1,   -1,   -1,  295,   -1,  256,  257,
  258,  259,   -1,  261,  262,   -1,   -1,  265,  266,  267,
  268,   -1,  270,  271,   -1,  273,  274,  275,  276,   -1,
   -1,  279,   -1,   -1,  282,   -1,   -1,   -1,  286,  287,
  288,  289,   -1,  291,    6,  293,   -1,  295,   10,   11,
   -1,   13,  300,   -1,  302,   -1,  304,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   30,   -1,
   -1,   -1,  125,  258,  259,   37,   -1,   -1,   -1,   -1,
  265,   -1,   44,  268,   -1,   -1,  271,   -1,   -1,  274,
   -1,  276,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  289,   -1,  291,   -1,   -1,   -1,
  295,   -1,   74,  125,   -1,   77,   78,   -1,   80,   81,
   82,   83,   84,   85,   -1,   -1,   88,   89,   -1,   -1,
   92,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  108,   -1,   -1,  111,
  112,   -1,   -1,   -1,   -1,   -1,  118,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  130,   -1,
  132,   -1,   -1,   -1,  136,   -1,  138,   -1,   -1,  141,
   -1,   -1,  144,  145,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  159,  160,  161,
  162,   -1,  164,  256,  257,  258,  259,   -1,  261,  262,
   -1,   -1,  265,  266,  267,  268,   -1,  179,  271,   -1,
  273,  274,  275,  276,   -1,   -1,  279,   -1,   -1,  282,
   -1,   -1,   -1,   -1,  287,   -1,  289,   -1,  291,   -1,
   -1,   -1,  295,   -1,  256,  257,  258,  259,   -1,  261,
  262,   -1,   -1,  265,  266,  267,  268,   -1,   -1,  271,
   -1,  273,  274,  275,  276,   -1,   -1,  279,   -1,   -1,
  282,   -1,   -1,   -1,   -1,  287,   -1,  289,   -1,  291,
   -1,   -1,   -1,  295,
};
#define YYFINAL 1
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 305
#if YYDEBUG
const char * const yyname[] =
	{
"end-of-file",0,0,0,0,0,0,0,0,0,"'\\n'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"';'",0,"'='",0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"ALIAS",
"AUTO","BLOCK","CA","CERT","CGI","CHROOT","CLIENT","DEFAULT","ENTRYPOINT","ENV",
"FASTCGI","FOR_HOST","INCLUDE","INDEX","IPV6","KEY","LANG","LOCATION","LOG",
"MAP","MIME","OCSP","OFF","ON","PARAM","PORT","PREFORK","PROTO","PROTOCOLS",
"PROXY","RELAY_TO","REQUIRE","RETURN","ROOT","SERVER","SNI","SPAWN","STRIP",
"TCP","TOEXT","TYPE","TYPES","USE_TLS","USER","VERIFYNAME","ERROR","STRING",
"NUM",
};
const char * const yyrule[] =
	{"$accept : conf",
"conf :",
"conf : conf include '\\n'",
"conf : conf '\\n'",
"conf : conf varset '\\n'",
"conf : conf option '\\n'",
"conf : conf vhost '\\n'",
"conf : conf types '\\n'",
"conf : conf error '\\n'",
"include : INCLUDE STRING",
"bool : ON",
"bool : OFF",
"string : string STRING",
"string : STRING",
"numberstring : NUM",
"numberstring : STRING",
"varset : STRING '=' string",
"option : CHROOT string",
"option : IPV6 bool",
"option : MIME STRING string",
"option : MAP string TOEXT string",
"option : PORT NUM",
"option : PREFORK NUM",
"option : PROTOCOLS string",
"option : USER string",
"$$1 :",
"vhost : SERVER string $$1 '{' optnl servbody '}'",
"vhost : error '}'",
"servbody :",
"servbody : servbody servopt optnl",
"servbody : servbody location optnl",
"servbody : servbody proxy optnl",
"servopt : ALIAS string",
"servopt : CERT string",
"servopt : CGI string",
"servopt : ENTRYPOINT string",
"servopt : ENV string '=' string",
"servopt : KEY string",
"servopt : OCSP string",
"servopt : PARAM string '=' string",
"servopt : locopt",
"$$2 :",
"proxy : PROXY $$2 proxy_matches '{' optnl proxy_opts '}'",
"proxy_matches :",
"proxy_matches : proxy_matches proxy_match",
"proxy_match : PROTO string",
"proxy_match : FOR_HOST string",
"proxy_opts :",
"proxy_opts : proxy_opts proxy_opt optnl",
"proxy_opt : CERT string",
"proxy_opt : KEY string",
"proxy_opt : PROTOCOLS string",
"proxy_opt : RELAY_TO string",
"proxy_opt : REQUIRE CLIENT CA string",
"proxy_opt : SNI string",
"proxy_opt : USE_TLS bool",
"proxy_opt : VERIFYNAME bool",
"$$3 :",
"location : LOCATION $$3 string '{' optnl locopts '}'",
"location : error '}'",
"locopts :",
"locopts : locopts locopt optnl",
"locopt : AUTO INDEX bool",
"locopt : BLOCK RETURN NUM string",
"locopt : BLOCK RETURN NUM",
"locopt : BLOCK",
"locopt : DEFAULT TYPE string",
"locopt : FASTCGI fastcgi",
"locopt : INDEX string",
"locopt : LANG string",
"locopt : LOG bool",
"locopt : REQUIRE CLIENT CA string",
"locopt : ROOT string",
"locopt : STRIP NUM",
"fastcgi : SPAWN string",
"fastcgi : string",
"fastcgi : TCP string PORT NUM",
"fastcgi : TCP string",
"fastcgi : TCP string PORT string",
"types : TYPES '{' optnl mediaopts_l '}'",
"mediaopts_l : mediaopts_l mediaoptsl nl",
"mediaopts_l : mediaoptsl nl",
"$$4 :",
"mediaoptsl : STRING $$4 medianames_l optsemicolon",
"mediaoptsl : include",
"medianames_l : medianames_l medianamesl",
"medianames_l : medianamesl",
"medianamesl : numberstring",
"nl : '\\n' optnl",
"optnl : '\\n' optnl",
"optnl : ';' optnl",
"optnl :",
"optsemicolon : ';'",
"optsemicolon :",
};
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
/* LINTUSED */
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
unsigned int yystacksize;
int yyparse(void);
#line 514 "parse.y"

static const struct keyword {
	const char *word;
	int token;
} keywords[] = {
	/* these MUST be sorted */
	{"alias", ALIAS},
	{"auto", AUTO},
	{"block", BLOCK},
	{"ca", CA},
	{"cert", CERT},
	{"cgi", CGI},
	{"chroot", CHROOT},
	{"client", CLIENT},
	{"default", DEFAULT},
	{"entrypoint", ENTRYPOINT},
	{"env", ENV},
	{"fastcgi", FASTCGI},
	{"for-host", FOR_HOST},
	{"include", INCLUDE},
	{"index", INDEX},
	{"ipv6", IPV6},
	{"key", KEY},
	{"lang", LANG},
	{"location", LOCATION},
	{"log", LOG},
	{"map", MAP},
	{"mime", MIME},
	{"ocsp", OCSP},
	{"off", OFF},
	{"on", ON},
	{"param", PARAM},
	{"port", PORT},
	{"prefork", PREFORK},
	{"proto", PROTO},
	{"protocols", PROTOCOLS},
	{"proxy", PROXY},
	{"relay-to", RELAY_TO},
	{"require", REQUIRE},
	{"return", RETURN},
	{"root", ROOT},
	{"server", SERVER},
	{"sni", SNI},
	{"spawn", SPAWN},
	{"strip", STRIP},
	{"tcp", TCP},
	{"to-ext", TOEXT},
	{"type", TYPE},
	{"types", TYPES},
	{"use-tls", USE_TLS},
	{"user", USER},
	{"verifyname", VERIFYNAME},
};

void
yyerror(const char *msg, ...)
{
	va_list ap;

	file->errors++;

	va_start(ap, msg);
	fprintf(stderr, "%s:%d error: ", config_path, yylval.lineno);
	vfprintf(stderr, msg, ap);
	fprintf(stderr, "\n");
	va_end(ap);
}

void
yywarn(const char *msg, ...)
{
	va_list ap;

	va_start(ap, msg);
	fprintf(stderr, "%s:%d warning: ", config_path, yylval.lineno);
	vfprintf(stderr, msg, ap);
	fprintf(stderr, "\n");
	va_end(ap);
}

int
kw_cmp(const void *k, const void *e)
{
	return strcmp(k, ((struct keyword *)e)->word);
}

int
lookup(char *s)
{
	const struct keyword	*p;

	p = bsearch(s, keywords, sizeof(keywords)/sizeof(keywords[0]),
	    sizeof(keywords[0]), kw_cmp);

	if (p)
		return p->token;
	else
		return STRING;
}

#define START_EXPAND	1
#define DONE_EXPAND	2

static int	expanding;

int
igetc(void)
{
	int	c;

	while (1) {
		if (file->ungetpos > 0)
			c = file->ungetbuf[--file->ungetpos];
		else
			c = getc(file->stream);

		if (c == START_EXPAND)
			expanding = 1;
		else if (c == DONE_EXPAND)
			expanding = 0;
		else
			break;
	}
	return c;
}

int
lgetc(int quotec)
{
	int		c, next;

	if (quotec) {
		if ((c = igetc()) == EOF) {
			yyerror("reached end of file while parsing "
			    "quoted string");
			if (file == topfile || popfile() == EOF)
				return EOF;
			return quotec;
		}
		return c;
	}

	while ((c = igetc()) == '\\') {
		next = igetc();
		if (next != '\n') {
			c = next;
			break;
		}
		yylval.lineno = file->lineno;
		file->lineno++;
	}

	if (c == EOF) {
		/*
		 * Fake EOL when hit EOF for the first time. This gets line
		 * count right if last line in included file is syntactically
		 * invalid and has no newline.
		 */
		if (file->eof_reached == 0) {
			file->eof_reached = 1;
			return '\n';
		}
		while (c == EOF) {
			if (file == topfile || popfile() == EOF)
				return EOF;
			c = igetc();
		}
	}
	return c;
}

void
lungetc(int c)
{
	if (c == EOF)
		return;

	if (file->ungetpos >= file->ungetsize) {
		void *p = reallocarray(file->ungetbuf, file->ungetsize, 2);
		if (p == NULL)
			err(1, "lungetc");
		file->ungetbuf = p;
		file->ungetsize *= 2;
	}
	file->ungetbuf[file->ungetpos++] = c;
}

int
findeol(void)
{
	int	c;

	/* Skip to either EOF or the first real EOL. */
	while (1) {
		c = lgetc(0);
		if (c == '\n') {
			file->lineno++;
			break;
		}
		if (c == EOF)
			break;
	}
	return ERROR;
}

int
yylex(void)
{
	char	 buf[8096];
	char	*p, *val;
	int	 quotec, next, c;
	int	 token;

top:
	p = buf;
	while ((c = lgetc(0)) == ' ' || c == '\t')
		; /* nothing */

	yylval.lineno = file->lineno;
	if (c == '#')
		while ((c = lgetc(0)) != '\n' && c != EOF)
			; /* nothing */
	if (c == '$' && !expanding) {
		while (1) {
			if ((c = lgetc(0)) == EOF)
				return 0;
			if (p + 1 >= buf + sizeof(buf) -1) {
				yyerror("string too long");
				return findeol();
			}
			if (isalnum(c) || c == '_') {
				*p++ = c;
				continue;
			}
			*p = '\0';
			lungetc(c);
			break;
		}
		val = symget(buf);
		if (val == NULL) {
			yyerror("macro `%s' not defined", buf);
			return findeol();
		}
		yylval.v.string = xstrdup(val);
		return STRING;
	}
	if (c == '@' && !expanding) {
		while (1) {
			if ((c = lgetc(0)) == EOF)
				return 0;

			if (p + 1 >= buf + sizeof(buf) - 1) {
				yyerror("string too long");
				return findeol();
			}
			if (isalnum(c) || c == '_') {
				*p++ = c;
				continue;
			}
			*p = '\0';
			lungetc(c);
			break;
		}
		val = symget(buf);
		if (val == NULL) {
			yyerror("macro '%s' not defined", buf);
			return findeol();
		}
		p = val + strlen(val) - 1;
		lungetc(DONE_EXPAND);
		while (p >= val) {
			lungetc(*p);
			p--;
		}
		lungetc(START_EXPAND);
		goto top;
	}

	switch (c) {
	case '\'':
	case '"':
		quotec = c;
		while (1) {
			if ((c = lgetc(quotec)) == EOF)
				return 0;
			if (c == '\n') {
				file->lineno++;
				continue;
			} else if (c == '\\') {
				if ((next = lgetc(quotec)) == EOF)
					return (0);
				if (next == quotec || next == ' ' ||
				    next == '\t')
					c = next;
				else if (next == '\n') {
					file->lineno++;
					continue;
				} else
					lungetc(next);
			} else if (c == quotec) {
				*p = '\0';
				break;
			} else if (c == '\0') {
				yyerror("invalid syntax");
				return findeol();
			}
			if (p + 1 >= buf + sizeof(buf) - 1) {
				yyerror("string too long");
				return findeol();
			}
			*p++ = c;
		}
		yylval.v.string = strdup(buf);
		if (yylval.v.string == NULL)
			err(1, "yylex: strdup");
		return STRING;
	}

#define allowed_to_end_number(x) \
	(isspace(x) || x == ')' || x ==',' || x == '/' || x == '}' || x == '=')

	if (c == '-' || isdigit(c)) {
		do {
			*p++ = c;
			if ((size_t)(p-buf) >= sizeof(buf)) {
				yyerror("string too long");
				return findeol();
			}
		} while ((c = lgetc(0)) != EOF && isdigit(c));
		lungetc(c);
		if (p == buf + 1 && buf[0] == '-')
			goto nodigits;
		if (c == EOF || allowed_to_end_number(c)) {
			const char *errstr = NULL;

			*p = '\0';
			yylval.v.number = strtonum(buf, LLONG_MIN,
			    LLONG_MAX, &errstr);
			if (errstr) {
				yyerror("\"%s\" invalid number: %s",
				    buf, errstr);
				return findeol();
			}
			return NUM;
		} else {
nodigits:
			while (p > buf + 1)
				lungetc(*--p);
			c = *--p;
			if (c == '-')
				return c;
		}
	}

#define allowed_in_string(x) \
	(isalnum(x) || (ispunct(x) && x != '(' && x != ')' && \
	x != '{' && x != '}' && \
	x != '!' && x != '=' && x != '#' && \
	x != ',' && x != ';'))

	if (isalnum(c) || c == ':' || c == '_') {
		do {
			*p++ = c;
			if ((size_t)(p-buf) >= sizeof(buf)) {
				yyerror("string too long");
				return findeol();
			}
		} while ((c = lgetc(0)) != EOF && (allowed_in_string(c)));
		lungetc(c);
		*p = '\0';
		if ((token = lookup(buf)) == STRING)
			yylval.v.string = xstrdup(buf);
		return token;
	}
	if (c == '\n') {
		yylval.lineno = file->lineno;
		file->lineno++;
	}
	if (c == EOF)
		return 0;
	return c;
}

struct file *
pushfile(const char *name, int secret)
{
	struct file	*nfile;

	nfile = xcalloc(1, sizeof(*nfile));
	nfile->name = xstrdup(name);
	if ((nfile->stream = fopen(nfile->name, "r")) == NULL) {
		log_warn(NULL, "can't open %s: %s", nfile->name,
		    strerror(errno));
		free(nfile->name);
		free(nfile);
		return NULL;
	}
	nfile->lineno = TAILQ_EMPTY(&files) ? 1 : 0;
	nfile->ungetsize = 16;
	nfile->ungetbuf = xcalloc(1, nfile->ungetsize);
	TAILQ_INSERT_TAIL(&files, nfile, entry);
	return nfile;
}

int
popfile(void)
{
	struct file	*prev;

	if ((prev = TAILQ_PREV(file, files, entry)) != NULL)
		prev->errors += file->errors;

	TAILQ_REMOVE(&files, file, entry);
	fclose(file->stream);
	free(file->name);
	free(file->ungetbuf);
	free(file);
	file = prev;
	return file ? 0 : EOF;
}

void
parse_conf(const char *filename)
{
	struct sym		*sym, *next;

	file = pushfile(filename, 0);
	if (file == NULL)
		exit(1);
	topfile = file;

	yyparse();
	errors = file->errors;
	popfile();

	/* Free macros and check which have not been used. */
	TAILQ_FOREACH_SAFE(sym, &symhead, entry, next) {
		/* TODO: warn if !sym->used */
		if (!sym->persist) {
			free(sym->name);
			free(sym->val);
			TAILQ_REMOVE(&symhead, sym, entry);
			free(sym);
		}
	}

	if (errors)
		exit(1);
}

void
print_conf(void)
{
	struct vhost	*h;
	/* struct location	*l; */
	/* struct envlist	*e; */
	/* struct alist	*a; */

	if (conf.chroot != NULL)
		printf("chroot \"%s\"\n", conf.chroot);
	printf("ipv6 %s\n", conf.ipv6 ? "on" : "off");
	/* XXX: defined mimes? */
	printf("port %d\n", conf.port);
	printf("prefork %d\n", conf.prefork);
	/* XXX: protocols? */
	if (conf.user != NULL)
		printf("user \"%s\"\n", conf.user);

	TAILQ_FOREACH(h, &hosts, vhosts) {
		printf("\nserver \"%s\" {\n", h->domain);
		printf("	cert \"%s\"\n", h->cert);
		printf("	key \"%s\"\n", h->key);
		/* TODO: print locations... */
		printf("}\n");
	}
}

int
symset(const char *name, const char *val, int persist)
{
	struct sym *sym;

	TAILQ_FOREACH(sym, &symhead, entry) {
		if (!strcmp(name, sym->name))
			break;
	}

	if (sym != NULL) {
		if (sym->persist)
			return 0;
		else {
			free(sym->name);
			free(sym->val);
			TAILQ_REMOVE(&symhead, sym, entry);
			free(sym);
		}
	}

	sym = xcalloc(1, sizeof(*sym));
	sym->name = xstrdup(name);
	sym->val = xstrdup(val);
	sym->used = 0;
	sym->persist = persist;

	TAILQ_INSERT_TAIL(&symhead, sym, entry);
	return 0;
}

int
cmdline_symset(char *s)
{
	char	*sym, *val;
	int	 ret;

	if ((val = strrchr(s, '=')) == NULL)
		return -1;
	sym = xcalloc(1, val - s + 1);
	memcpy(sym, s, val - s);
	ret = symset(sym, val + 1, 1);
	free(sym);
	return ret;
}

char *
symget(const char *nam)
{
	struct sym	*sym;

	TAILQ_FOREACH(sym, &symhead, entry) {
		if (strcmp(nam, sym->name) == 0) {
			sym->used = 1;
			return sym->val;
		}
	}
	return NULL;
}

struct vhost *
new_vhost(void)
{
	return xcalloc(1, sizeof(struct vhost));
}

struct location *
new_location(void)
{
	struct location *l;

	l = xcalloc(1, sizeof(*l));
	l->dirfd = -1;
	l->fcgi = -1;
	return l;
}

struct proxy *
new_proxy(void)
{
	struct proxy *p;

	p = xcalloc(1, sizeof(*p));
	p->protocols = TLS_PROTOCOLS_DEFAULT;
	return p;
}

char *
ensure_absolute_path(char *path)
{
	if (path == NULL || *path != '/')
		yyerror("not an absolute path: %s", path);
	return path;
}

int
check_block_code(int n)
{
	if (n < 10 || n >= 70 || (n >= 20 && n <= 29))
		yyerror("invalid block code %d", n);
	return n;
}

char *
check_block_fmt(char *fmt)
{
	char *s;

	for (s = fmt; *s; ++s) {
		if (*s != '%')
			continue;
		switch (*++s) {
		case '%':
		case 'p':
		case 'q':
		case 'P':
		case 'N':
			break;
		default:
			yyerror("invalid format specifier %%%c", *s);
		}
	}

	return fmt;
}

int
check_strip_no(int n)
{
	if (n <= 0)
		yyerror("invalid strip number %d", n);
	return n;
}

int
check_port_num(int n)
{
	if (n <= 0 || n >= UINT16_MAX)
		yyerror("port number is %s: %d",
		    n <= 0 ? "too small" : "too large",
		    n);
	return n;
}

int
check_prefork_num(int n)
{
	if (n <= 0 || n >= PROC_MAX)
		yyerror("invalid prefork number %d", n);
	return n;
}

void
advance_loc(void)
{
	loc = new_location();
	TAILQ_INSERT_TAIL(&host->locations, loc, locations);
}

void
advance_proxy(void)
{
	proxy = new_proxy();
	TAILQ_INSERT_TAIL(&host->proxies, proxy, proxies);
}

void
parsehp(char *str, char **host, const char **port, const char *def)
{
	char		*at;
	const char	*errstr;

	*host = str;

	if ((at = strchr(str, ':')) != NULL) {
		*at++ = '\0';
		*port = at;
	} else
		*port = def;

	strtonum(*port, 1, UINT16_MAX, &errstr);
	if (errstr != NULL)
		yyerror("port is %s: %s", errstr, *port);
}

void
only_once(const void *ptr, const char *name)
{
	if (ptr != NULL)
		yyerror("`%s' specified more than once", name);
}

void
only_oncei(int i, const char *name)
{
	if (i != -1)
		yyerror("`%s' specified more than once", name);
}

int
fastcgi_conf(char *path, char *port, char *prog)
{
	struct fcgi	*f;
	int		i;

	for (i = 0; i < FCGI_MAX; ++i) {
		f = &fcgi[i];

		if (f->path == NULL) {
			f->id = i;
			f->path = path;
			f->port = port;
			f->prog = prog;
			return i;
		}

		/* XXX: what to do with prog? */
		if (!strcmp(f->path, path) &&
		    ((port == NULL && f->port == NULL) ||
		     !strcmp(f->port, port))) {
			free(path);
			free(port);
			return i;
		}
	}

	yyerror("too much `fastcgi' rules defined.");
	return -1;
}

void
add_param(char *name, char *val, int env)
{
	struct envlist *e;
	struct envhead *h;

	if (env)
		h = &host->env;
	else
		h = &host->params;

	e = xcalloc(1, sizeof(*e));
	e->name = name;
	e->value = val;
	if (TAILQ_EMPTY(h))
		TAILQ_INSERT_HEAD(h, e, envs);
	else
		TAILQ_INSERT_TAIL(h, e, envs);
}
#line 1452 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(void)
{
    unsigned int newsize;
    long sslen;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    sslen = yyssp - yyss;
#ifdef SIZE_MAX
#define YY_SIZE_MAX SIZE_MAX
#else
#define YY_SIZE_MAX 0xffffffffU
#endif
    if (newsize && YY_SIZE_MAX / newsize < sizeof *newss)
        goto bail;
    newss = (short *)realloc(yyss, newsize * sizeof *newss);
    if (newss == NULL)
        goto bail;
    yyss = newss;
    yyssp = newss + sslen;
    if (newsize && YY_SIZE_MAX / newsize < sizeof *newvs)
        goto bail;
    newvs = (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs);
    if (newvs == NULL)
        goto bail;
    yyvs = newvs;
    yyvsp = newvs + sslen;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
bail:
    if (yyss)
            free(yyss);
    if (yyvs)
            free(yyvs);
    yyss = yyssp = NULL;
    yyvs = yyvsp = NULL;
    yystacksize = 0;
    return -1;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
int
yyparse(void)
{
    int yym, yyn, yystate;
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif /* YYDEBUG */

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym)
        yyval = yyvsp[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
    switch (yyn)
    {
case 8:
#line 153 "parse.y"
{ file->errors++; }
break;
case 9:
#line 156 "parse.y"
{
			struct file	*nfile;

			if ((nfile = pushfile(yyvsp[0].v.string, 0)) == NULL) {
				yyerror("failed to include file %s", yyvsp[0].v.string);
				free(yyvsp[0].v.string);
				YYERROR;
			}
			free(yyvsp[0].v.string);

			file = nfile;
			lungetc('\n');
		}
break;
case 10:
#line 171 "parse.y"
{ yyval.v.number = 1; }
break;
case 11:
#line 172 "parse.y"
{ yyval.v.number = 0; }
break;
case 12:
#line 175 "parse.y"
{
			if (asprintf(&yyval.v.string, "%s%s", yyvsp[-1].v.string, yyvsp[0].v.string) == -1) {
				free(yyvsp[-1].v.string);
				free(yyvsp[0].v.string);
				yyerror("string: asprintf: %s", strerror(errno));
				YYERROR;
			}
			free(yyvsp[-1].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 14:
#line 188 "parse.y"
{
			char *s;
			if (asprintf(&s, "%d", yyvsp[0].v.number) == -1) {
				yyerror("asprintf: number");
				YYERROR;
			}
			yyval.v.string = s;
		}
break;
case 16:
#line 199 "parse.y"
{
			char *s = yyvsp[-2].v.string;
			while (*s++) {
				if (isspace((unsigned char)*s)) {
					yyerror("macro name cannot contain "
					    "whitespaces");
					free(yyvsp[-2].v.string);
					free(yyvsp[0].v.string);
					YYERROR;
				}
			}
			symset(yyvsp[-2].v.string, yyvsp[0].v.string, 0);
			free(yyvsp[-2].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 17:
#line 216 "parse.y"
{ conf.chroot = yyvsp[0].v.string; }
break;
case 18:
#line 217 "parse.y"
{ conf.ipv6 = yyvsp[0].v.number; }
break;
case 19:
#line 218 "parse.y"
{
			yywarn("`mime MIME EXT' is deprecated and will be "
			    "removed in a future version, please use the new "
			    "`types' block.");
			if (add_mime(&conf.mime, yyvsp[-1].v.string, yyvsp[0].v.string) == -1)
				err(1, "add_mime");
		}
break;
case 20:
#line 225 "parse.y"
{
			yywarn("`map mime to-ext' is deprecated and will be "
			    "removed in a future version, please use the new "
			    "`types' block.");
			if (add_mime(&conf.mime, yyvsp[-2].v.string, yyvsp[0].v.string) == -1)
				err(1, "add_mime");
		}
break;
case 21:
#line 232 "parse.y"
{ conf.port = check_port_num(yyvsp[0].v.number); }
break;
case 22:
#line 233 "parse.y"
{ conf.prefork = check_prefork_num(yyvsp[0].v.number); }
break;
case 23:
#line 234 "parse.y"
{
			if (tls_config_parse_protocols(&conf.protos, yyvsp[0].v.string) == -1)
				yyerror("invalid protocols string \"%s\"", yyvsp[0].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 24:
#line 239 "parse.y"
{ conf.user = yyvsp[0].v.string; }
break;
case 25:
#line 242 "parse.y"
{
			host = new_vhost();
			TAILQ_INSERT_HEAD(&hosts, host, vhosts);

			loc = new_location();
			TAILQ_INSERT_HEAD(&host->locations, loc, locations);

			TAILQ_INIT(&host->proxies);

			loc->match = xstrdup("*");
			host->domain = yyvsp[0].v.string;

			if (strstr(yyvsp[0].v.string, "xn--") != NULL) {
				yywarn("\"%s\" looks like punycode: you "
				    "should use the decoded hostname", yyvsp[0].v.string);
			}
		}
break;
case 26:
#line 258 "parse.y"
{
			if (host->cert == NULL || host->key == NULL)
				yyerror("invalid vhost definition: %s", yyvsp[-5].v.string);
		}
break;
case 27:
#line 262 "parse.y"
{ yyerror("bad server directive"); }
break;
case 32:
#line 271 "parse.y"
{
			struct alist *a;

			a = xcalloc(1, sizeof(*a));
			a->alias = yyvsp[0].v.string;
			if (TAILQ_EMPTY(&host->aliases))
				TAILQ_INSERT_HEAD(&host->aliases, a, aliases);
			else
				TAILQ_INSERT_TAIL(&host->aliases, a, aliases);
		}
break;
case 33:
#line 281 "parse.y"
{
			only_once(host->cert, "cert");
			host->cert = ensure_absolute_path(yyvsp[0].v.string);
		}
break;
case 34:
#line 285 "parse.y"
{
			only_once(host->cgi, "cgi");
			/* drop the starting '/', if any */
			if (*yyvsp[0].v.string == '/')
				memmove(yyvsp[0].v.string, yyvsp[0].v.string+1, strlen(yyvsp[0].v.string));
			host->cgi = yyvsp[0].v.string;
		}
break;
case 35:
#line 292 "parse.y"
{
			only_once(host->entrypoint, "entrypoint");
			while (*yyvsp[0].v.string == '/')
				memmove(yyvsp[0].v.string, yyvsp[0].v.string+1, strlen(yyvsp[0].v.string));
			host->entrypoint = yyvsp[0].v.string;
		}
break;
case 36:
#line 298 "parse.y"
{
			add_param(yyvsp[-2].v.string, yyvsp[0].v.string, 1);
		}
break;
case 37:
#line 301 "parse.y"
{
			only_once(host->key, "key");
			host->key  = ensure_absolute_path(yyvsp[0].v.string);
		}
break;
case 38:
#line 305 "parse.y"
{
			only_once(host->ocsp, "ocsp");
			host->ocsp = ensure_absolute_path(yyvsp[0].v.string);
		}
break;
case 39:
#line 309 "parse.y"
{
			add_param(yyvsp[-2].v.string, yyvsp[0].v.string, 0);
		}
break;
case 41:
#line 315 "parse.y"
{ advance_proxy(); }
break;
case 42:
#line 316 "parse.y"
{
			if (proxy->host == NULL)
				yyerror("invalid proxy block: missing `relay-to' option");

			if ((proxy->cert == NULL && proxy->key != NULL) ||
			    (proxy->cert != NULL && proxy->key == NULL))
				yyerror("invalid proxy block: missing cert or key");
		}
break;
case 45:
#line 330 "parse.y"
{
			only_once(proxy->match_proto, "proxy proto");
			free(proxy->match_proto);
			proxy->match_proto = yyvsp[0].v.string;
		}
break;
case 46:
#line 335 "parse.y"
{
			only_once(proxy->match_host, "proxy for-host");
			free(proxy->match_host);
			parsehp(yyvsp[0].v.string, &proxy->match_host, &proxy->match_port, "10965");
		}
break;
case 49:
#line 346 "parse.y"
{
			only_once(proxy->cert, "proxy cert");
			tls_unload_file(proxy->cert, proxy->certlen);
			ensure_absolute_path(yyvsp[0].v.string);
			proxy->cert = tls_load_file(yyvsp[0].v.string, &proxy->certlen, NULL);
			if (proxy->cert == NULL)
				yyerror("can't load cert %s", yyvsp[0].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 50:
#line 355 "parse.y"
{
			only_once(proxy->key, "proxy key");
			tls_unload_file(proxy->key, proxy->keylen);
			ensure_absolute_path(yyvsp[0].v.string);
			proxy->key = tls_load_file(yyvsp[0].v.string, &proxy->keylen, NULL);
			if (proxy->key == NULL)
				yyerror("can't load key %s", yyvsp[0].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 51:
#line 364 "parse.y"
{
			if (tls_config_parse_protocols(&proxy->protocols, yyvsp[0].v.string) == -1)
				yyerror("invalid protocols string \"%s\"", yyvsp[0].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 52:
#line 369 "parse.y"
{
			only_once(proxy->host, "proxy relay-to");
			free(proxy->host);
			parsehp(yyvsp[0].v.string, &proxy->host, &proxy->port, "1965");
		}
break;
case 53:
#line 374 "parse.y"
{
			only_once(proxy->reqca, "require client ca");
			ensure_absolute_path(yyvsp[0].v.string);
			if ((proxy->reqca = load_ca(yyvsp[0].v.string)) == NULL)
				yyerror("couldn't load ca cert: %s", yyvsp[0].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 54:
#line 381 "parse.y"
{
			only_once(proxy->sni, "proxy sni");
			free(proxy->sni);
			proxy->sni = yyvsp[0].v.string;
		}
break;
case 55:
#line 386 "parse.y"
{
			proxy->notls = !yyvsp[0].v.number;
		}
break;
case 56:
#line 389 "parse.y"
{
			proxy->noverifyname = !yyvsp[0].v.number;
		}
break;
case 57:
#line 394 "parse.y"
{ advance_loc(); }
break;
case 58:
#line 394 "parse.y"
{
			/* drop the starting '/' if any */
			if (*yyvsp[-4].v.string == '/')
				memmove(yyvsp[-4].v.string, yyvsp[-4].v.string+1, strlen(yyvsp[-4].v.string));
			loc->match = yyvsp[-4].v.string;
		}
break;
case 62:
#line 407 "parse.y"
{ loc->auto_index = yyvsp[0].v.number ? 1 : -1; }
break;
case 63:
#line 408 "parse.y"
{
			only_once(loc->block_fmt, "block");
			loc->block_fmt = check_block_fmt(yyvsp[0].v.string);
			loc->block_code = check_block_code(yyvsp[-1].v.number);
		}
break;
case 64:
#line 413 "parse.y"
{
			only_once(loc->block_fmt, "block");
			loc->block_fmt = xstrdup("temporary failure");
			loc->block_code = check_block_code(yyvsp[0].v.number);
			if (yyvsp[0].v.number >= 30 && yyvsp[0].v.number < 40)
				yyerror("missing `meta' for block return %d", yyvsp[0].v.number);
		}
break;
case 65:
#line 420 "parse.y"
{
			only_once(loc->block_fmt, "block");
			loc->block_fmt = xstrdup("temporary failure");
			loc->block_code = 40;
		}
break;
case 66:
#line 425 "parse.y"
{
			only_once(loc->default_mime, "default type");
			loc->default_mime = yyvsp[0].v.string;
		}
break;
case 68:
#line 430 "parse.y"
{
			only_once(loc->index, "index");
			loc->index = yyvsp[0].v.string;
		}
break;
case 69:
#line 434 "parse.y"
{
			only_once(loc->lang, "lang");
			loc->lang = yyvsp[0].v.string;
		}
break;
case 70:
#line 438 "parse.y"
{ loc->disable_log = !yyvsp[0].v.number; }
break;
case 71:
#line 439 "parse.y"
{
			only_once(loc->reqca, "require client ca");
			ensure_absolute_path(yyvsp[0].v.string);
			if ((loc->reqca = load_ca(yyvsp[0].v.string)) == NULL)
				yyerror("couldn't load ca cert: %s", yyvsp[0].v.string);
			free(yyvsp[0].v.string);
		}
break;
case 72:
#line 446 "parse.y"
{
			only_once(loc->dir, "root");
			loc->dir  = ensure_absolute_path(yyvsp[0].v.string);
		}
break;
case 73:
#line 450 "parse.y"
{ loc->strip = check_strip_no(yyvsp[0].v.number); }
break;
case 74:
#line 453 "parse.y"
{
			only_oncei(loc->fcgi, "fastcgi");
			loc->fcgi = fastcgi_conf(NULL, NULL, yyvsp[0].v.string);
		}
break;
case 75:
#line 457 "parse.y"
{
			only_oncei(loc->fcgi, "fastcgi");
			loc->fcgi = fastcgi_conf(yyvsp[0].v.string, NULL, NULL);
		}
break;
case 76:
#line 461 "parse.y"
{
			char *c;
			if (asprintf(&c, "%d", yyvsp[0].v.number) == -1)
				err(1, "asprintf");
			only_oncei(loc->fcgi, "fastcgi");
			loc->fcgi = fastcgi_conf(yyvsp[-2].v.string, c, NULL);
		}
break;
case 77:
#line 468 "parse.y"
{
			only_oncei(loc->fcgi, "fastcgi");
			loc->fcgi = fastcgi_conf(yyvsp[0].v.string, xstrdup("9000"), NULL);
		}
break;
case 78:
#line 472 "parse.y"
{
			only_oncei(loc->fcgi, "fastcgi");
			loc->fcgi = fastcgi_conf(yyvsp[-2].v.string, yyvsp[0].v.string, NULL);
		}
break;
case 79:
#line 478 "parse.y"
{
			conf.mime.skip_defaults = 1;
		}
break;
case 82:
#line 487 "parse.y"
{ current_media = yyvsp[0].v.string; }
break;
case 87:
#line 495 "parse.y"
{
			if (add_mime(&conf.mime, current_media, yyvsp[0].v.string) == -1)
				err(1, "add_mime");
		}
break;
#line 2103 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    if (yyss)
            free(yyss);
    if (yyvs)
            free(yyvs);
    yyss = yyssp = NULL;
    yyvs = yyvsp = NULL;
    yystacksize = 0;
    return (1);
yyaccept:
    if (yyss)
            free(yyss);
    if (yyvs)
            free(yyvs);
    yyss = yyssp = NULL;
    yyvs = yyvsp = NULL;
    yystacksize = 0;
    return (0);
}
